<?php
// This file is part of VPL for Moodle - http://vpl.dis.ulpgc.es/
//
// VPL for Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VPL for Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VPL for Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * IDE utility functions
 *
 * @package mod_vpl
 * @copyright 2012 Juan Carlos Rodríguez-del-Pino
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @author Juan Carlos Rodríguez-del-Pino <jcrodriguez@dis.ulpgc.es>
 */

defined( 'MOODLE_INTERNAL' ) || die();
class vpl_editor_util {
    public static function generate_jquery() {
        global $PAGE;
        $PAGE->requires->jquery();
        $PAGE->requires->jquery_plugin('ui');
        $PAGE->requires->jquery_plugin('ui-css');
        $PAGE->requires->jquery_plugin('ui-touch', 'mod_vpl');
    }
    public static function generate_requires_evaluation() {
        self::generate_jquery();
    }
    public static function generate_requires($options) {
        global $PAGE;
        $plugincfg = get_config('mod_vpl');
        if ( isset($plugincfg->editor_theme) ) {
            $options ['theme'] = $plugincfg->editor_theme;
        } else {
            $options ['theme'] = 'chrome';
        }
        $options ['fontSize'] = get_user_preferences('vpl_editor_fontsize', 12);
        $options ['theme'] = get_user_preferences('vpl_acetheme', $options ['theme']);
        $options ['str'] = self::strings_for_js();
        self::generate_jquery();
        $PAGE->requires->js( new moodle_url( '/mod/vpl/editor/zip/inflate.js' ) );
        $PAGE->requires->js( new moodle_url( '/mod/vpl/editor/zip/unzip.js' ) );
        $PAGE->requires->js( new moodle_url( '/mod/vpl/editor/xterm/term.js' ) );
        $PAGE->requires->js( new moodle_url( '/mod/vpl/editor/noVNC/include/util.js' ) );
        $PAGE->requires->js_call_amd('mod_vpl/vplide', 'init', array('vplide', $options));

        $PAGE->requires->js_init_call('M.util.init_colour_picker', array('interfacetheme-colorpicker-primary'));
        $PAGE->requires->js_init_call('M.util.init_colour_picker', array('interfacetheme-colorpicker-secondary'));
        echo '<style type="text/css" title="custom-interfacetheme-css"></style>';
    }

    public static function print_editor() {
        global $OUTPUT;
        $context = new stdClass();
        $context->currenttheme = get_user_preferences('vpl_interfacetheme', 'modernblue');
        $context->currentprimarycolor = get_user_preferences('vpl_custom_interface_theme_color_primary', '#000000');
        $context->currentsecondarycolor = get_user_preferences('vpl_custom_interface_theme_color_secondary', '#FFFFFF');
        $context->installedthemes = array();
        foreach (self::get_installed_themes() as $id => $name) {
            $theme = new stdClass();
            $theme->id = $id;
            $theme->name = $name;
            $context->installedthemes[] = $theme;
        }
        echo $OUTPUT->box($OUTPUT->render_from_template('mod_vpl/editor', $context));
    }

    public static function strings_for_js() {
        global $PAGE;
        $PAGE->requires->strings_for_js(
                array (
                        'about',
                        'acceptcertificates',
                        'acceptcertificatesnote',
                        'acetheme',
                        'binaryfile',
                        'browserupdate',
                        'changesNotSaved',
                        'clipboard',
                        'comments',
                        'compilation',
                        'connected',
                        'connecting',
                        'connection_closed',
                        'connection_fail',
                        'console',
                        'copy',
                        'correctedfiles',
                        'create_new_file',
                        'cut',
                        'description',
                        'debug',
                        'debugging',
                        'delete',
                        'delete_file_fq',
                        'delete_file_q',
                        'download',
                        'edit',
                        'evaluate',
                        'evaluating',
                        'execution',
                        'getjails',
                        'file',
                        'filelist',
                        'filenotadded',
                        'filenotdeleted',
                        'filenotrenamed',
                        'find',
                        'find_replace',
                        'fullscreen',
                        'incorrect_file_name',
                        'interfacetheme',
                        'keyboard',
                        'maxfilesexceeded',
                        'morerecentversionloadprompt',
                        'morerecentversionsaved',
                        'new',
                        'next',
                        'load',
                        'loading',
                        'options',
                        'outofmemory',
                        'paste',
                        'print',
                        'redo',
                        'regularscreen',
                        'rename',
                        'rename_file',
                        'resetfiles',
                        'retrieve',
                        'run',
                        'running',
                        'save',
                        'saving',
                        'select_all',
                        'shortcuts',
                        'sureresetfiles',
                        'surecorrectedfiles',
                        'timeleft',
                        'timeout',
                        'undo',
                        'multidelete',
                        'basic',
                        'intermediate',
                        'advanced',
                        'variables',
                        'operatorsvalues',
                        'control',
                        'inputoutput',
                        'functions',
                        'lists',
                        'math',
                        'text',
                        'start',
                        'startanimate',
                        'stop',
                        'pause',
                        'resume',
                        'step'
                ), 'mod_vpl');
        $PAGE->requires->strings_for_js(
                array(
                        'cancel',
                        'closebuttontitle',
                        'error',
                        'import',
                        'modified',
                        'no',
                        'notice',
                        'ok',
                        'required',
                        'sort',
                        'warning',
                        'yes',
                        'deleteselected',
                        'selectall',
                        'deselectall',
                        'reset',
                        'theme'
                ), 'moodle');
        return array(
                'close' => get_string( 'closebuttontitle' ),
                'more' => get_string( 'showmore', 'form' ),
                'less' => get_string( 'showless', 'form' ),
                'fontsize' => get_string( 'fontsize', 'editor' )
        );
    }

    public static function generate_evaluate_script($ajaxurl, $nexturl) {
        global $PAGE;
        $options = array();
        $options ['str'] = self::strings_for_js();
        $options ['ajaxurl'] = $ajaxurl;
        $options ['nexturl'] = $nexturl;
        $PAGE->requires->js_call_amd('mod_vpl/evaluationmonitor', 'init', array($options) );
    }

    public static function get_installed_themes() {
        // Search for theme files.
        global $CFG;
        $acefiles = array_diff(scandir($CFG->dirroot . '/mod/vpl/editor/ace9/'), array('.', '..'));
        $themefiles = array_filter($acefiles, function($name) {
            return substr($name, 0, 6) == 'theme-';
        });
        // Process theme files names to get displayable name,
        // by replacing underscores by spaces and
        // by putting upper case letters at the beginning of words.
        $themes = array();
        foreach ($themefiles as $themefile) {
            $theme = substr($themefile, 6, strlen($themefile) - 9);
            $themename = preg_replace_callback('/(^|_)([a-z])/', function($matches) {
                return ' ' . strtoupper($matches[2]);
            }, $theme);
            $themes[$theme] = trim($themename);
        }
        // Some exceptions.
        $themes['github'] = 'GitHub';
        $themes['idle_fingers'] = 'idle Fingers';
        $themes['iplastic'] = 'IPlastic';
        $themes['katzenmilch'] = 'KatzenMilch';
        $themes['kr_theme'] = 'krTheme';
        $themes['kr'] = 'kr';
        $themes['pastel_on_dark'] = 'Pastel on dark';
        $themes['sqlserver'] = 'SQL Server';
        $themes['textmate'] = 'TextMate';
        $themes['tomorrow_night_eighties'] = 'Tomorrow Night 80s';
        $themes['xcode'] = 'XCode';
        return $themes;
    }
}
