<?php
// This file is part of VPL for Moodle - http://vpl.dis.ulpgc.es/
//
// VPL for Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VPL for Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VPL for Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Execution options form
 *
 * @package mod_vpl
 * @copyright 2012 Juan Carlos Rodríguez-del-Pino
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @author Juan Carlos Rodríguez-del-Pino <jcrodriguez@dis.ulpgc.es>
 */

require_once(dirname(__FILE__).'/../../../config.php');
require_once(dirname(__FILE__).'/../locallib.php');
require_once(dirname(__FILE__).'/../vpl.class.php');
global $CFG;
require_once($CFG->libdir.'/formslib.php');

class mod_vpl_executionoptions_form extends moodleform {
    protected $vpl;
    public function __construct($page, $vpl) {
        $this->vpl = $vpl;
        parent::__construct( $page );
    }
    protected function get_scriptdescription($filename) {
        $data = file_get_contents($filename);
        if ($data === false ) {
            return '';
        }
        $matches = [];
        $result = preg_match('/@vpl_script_description (.*)$/im', $data, $matches);
        if ( $result ) {
            return ': ' . $matches[1];
        }
        return '';
    }
    protected function get_dirlist($dir, $endwith) {
        $avoid = array('default' => 1);
        $el = strlen($endwith);
        $dirlist = scandir($dir);
        $list = array();
        foreach ($dirlist as $file) {
            if ( substr($file, - $el) == $endwith) {
                $name = substr($file, 0, - $el);
                if ( ! isset( $avoid[$name] ) ) {
                    $list[$name] = strtoupper($name) . $this->get_scriptdescription($dir . '/' . $file);
                }
            }
        }
        return $list;
    }

    protected function get_runlist() {
        return $this->get_dirlist('../jail/default_scripts', '_run.sh');
    }

    protected function get_debuglist() {
        return $this->get_dirlist('../jail/default_scripts', '_debug.sh');
    }

    protected function definition() {
        global $DB, $OUTPUT;
        $mform = & $this->_form;
        $id = $this->vpl->get_course_module()->id;
        $instance = $this->vpl->get_instance();
        $vplid = $instance->id;
        $mform->addElement( 'hidden', 'id', $id );
        $mform->setType( 'id', PARAM_INT );
        $mform->addElement( 'header', 'header_execution_options', get_string( 'executionoptions', VPL ) );
        $mform->addElement('advcheckbox', 'usableasbase', get_string('usableasbase', VPL));
        $mform->setType('usableasbase', PARAM_BOOL);
        $mform->setDefault('usableasbase', $instance->usableasbase);
        $strbasedon = get_string( 'basedon', VPL );
        $basedonlist = array ();
        $basedonlist[''] = array( get_string( 'select' ) );
        $basedonlist[$this->vpl->get_course()->id] = array();
        $courseslist = array_keys(enrol_get_my_courses());
        foreach ($courseslist as $courseid) {
            $listcm = get_coursemodules_in_course( VPL, $courseid );
            $basedonlist[$courseid] = array();
            foreach ($listcm as $aux) {
                if ($aux->instance != $vplid) {
                    $vpl = new mod_vpl( $aux->id );
                    if ($vpl->is_use_as_base()) {
                        $basedonlist[$courseid][$aux->instance] = $vpl->get_printable_name();
                    }
                }
            }
            asort($basedonlist[$courseid]);
        }

        foreach ($basedonlist as $courseid => $vpls) {
            if ($courseid && $vpls) {
                $basedonlist[get_string('fromcourse', VPL, get_course($courseid)->shortname)] = $vpls;
                unset($basedonlist[$courseid]);
            }
        }

        $metadataplugin = core_component::get_plugin_directory('local', 'metadata');
        if ($metadataplugin !== null) {
            $shared = get_string('shared', VPL);
            $basedonlist[$shared] = array();
            $vplmoduleid = $DB->get_record('modules', array('name' => 'vpl'), 'id')->id;
            $sharedfieldid = $DB->get_record('local_metadata_field', array('shortname' => 'Shared'), 'id')->id;
            $sharedvpls = $DB->get_records_sql("SELECT {course_modules}.id, {course_modules}.instance
                FROM {course_modules}
                JOIN {vpl} ON {vpl}.id = {course_modules}.instance
                JOIN {local_metadata} ON {course_modules}.id = {local_metadata}.instanceid
                WHERE {vpl}.usableasbase = 1 AND {local_metadata}.fieldid = $sharedfieldid
                    AND {local_metadata}.data = 1 AND {course_modules}.module = $vplmoduleid");
            foreach ($sharedvpls as $sharedvpl) {
                if ($sharedvpl->instance != $vplid) {
                    $vpl = new mod_vpl( $sharedvpl->id );
                    $basedonlist[$shared][$sharedvpl->instance] = $vpl->get_printable_name();
                }
            }
            asort($basedonlist[$shared]);
        }

        if ($instance->basedon) {
            // Search if current based on VPL is in the list.
            // If not, add it to the list to preserve its value.
            $hasaccesstocurrent = false;
            foreach ($basedonlist as $vpls) {
                if (array_key_exists($instance->basedon, $vpls)) {
                    $hasaccesstocurrent = true;
                    break;
                }
            }
            if (!$hasaccesstocurrent) {
                $basedonvpl = new mod_vpl(null, $instance->basedon);
                $basedonlist[''][$instance->basedon] = $basedonvpl->get_printable_name();
                $warningicon = $OUTPUT->pix_icon('warning', get_string('basedonwarning', VPL), 'mod_vpl');
                $strbasedon .= '<span class="float-sm-right text-warning">' . $warningicon . '</span>';
            }
        }

        $mform->addElement( 'selectgroups', 'basedon', $strbasedon, $basedonlist );
        $mform->setDefault( 'basedon', $instance->basedon );

        $strautodetect = get_string('autodetect', VPL);
        $strrunscript = get_string('runscript', VPL);
        $runlist = array_merge(array('' => $strautodetect), $this->get_runlist());
        $mform->addElement( 'select', 'runscript', $strrunscript, $runlist );
        $mform->setDefault( 'runscript', $instance->runscript );
        $mform->addHelpButton('runscript', 'runscript', VPL);

        $strdebugscript = get_string('debugscript', VPL);
        $debuglist = array_merge(array('' => $strautodetect), $this->get_debuglist());
        $mform->addElement( 'select', 'debugscript', $strdebugscript, $debuglist );
        $mform->setDefault( 'debugscript', $instance->debugscript );
        $mform->addHelpButton('debugscript', 'debugscript', VPL);

        $mform->addElement( 'selectyesno', 'run', get_string( 'run', VPL ) );
        $mform->setDefault( 'run', $instance->run );
        $mform->addElement( 'selectyesno', 'debug', get_string( 'debug', VPL ) );
        $mform->setDefault( 'debug', $instance->debug );
        $mform->addElement( 'selectyesno', 'evaluate', get_string( 'evaluate', VPL ) );
        $mform->setDefault( 'evaluate', $instance->evaluate );
        $mform->addElement( 'selectyesno', 'evaluateonsubmission', get_string( 'evaluateonsubmission', VPL ) );
        $mform->setDefault( 'evaluateonsubmission', $instance->evaluateonsubmission );
        $mform->disabledIf( 'evaluateonsubmission', 'evaluate', 'eq', 0 );
        $mform->addHelpButton( 'evaluateonsubmission', 'evaluateonsubmission', VPL );
        $mform->addElement( 'selectyesno', 'automaticgrading', get_string( 'automaticgrading', VPL ) );
        $mform->setDefault( 'automaticgrading', $instance->automaticgrading );
        $mform->disabledIf( 'automaticgrading', 'evaluate', 'eq', 0 );
        $mform->addHelpButton( 'automaticgrading', 'automaticgrading', VPL );

        $mform->addElement( 'submit', 'saveoptions', get_string( 'saveoptions', VPL ) );
    }
}

require_login();

$id = required_param( 'id', PARAM_INT );
$vpl = new mod_vpl( $id );
$vpl->prepare_page( 'forms/executionoptions.php', array ( 'id' => $id ) );
$vpl->require_capability( VPL_MANAGE_CAPABILITY );
global $PAGE;
$PAGE->force_settings_menu();
// Display page.
$vpl->print_header( get_string( 'executionoptions', VPL ) );
$vpl->print_heading_with_help( 'executionoptions' );

$mform = new mod_vpl_executionoptions_form( 'executionoptions.php', $vpl );
if ($fromform = $mform->get_data()) {
    if (isset( $fromform->saveoptions )) {
        $instance = $vpl->get_instance();
        \mod_vpl\event\vpl_execution_options_updated::log( $vpl );
        $instance->usableasbase = $fromform->usableasbase;
        $instance->basedon = $fromform->basedon;
        $instance->runscript = $fromform->runscript;
        $instance->debugscript = $fromform->debugscript;
        $instance->run = $fromform->run;
        $instance->debug = $fromform->debug;
        $instance->evaluate = $fromform->evaluate;
        $instance->evaluateonsubmission = $fromform->evaluate && $fromform->evaluateonsubmission;
        $instance->automaticgrading = $fromform->evaluate && $fromform->automaticgrading;
        if ( $vpl->update() ) {
            vpl_redirect( vpl_mod_href('view.php', 'id', $id), get_string( 'optionssaved', VPL ), 'success' );
        } else {
            vpl_notice( get_string( 'optionsnotsaved', VPL ), 'error' );
        }
    }
}
\mod_vpl\event\vpl_execution_options_viewed::log( $vpl );
$mform->display();
$vpl->print_footer();
