<?php
// This file is part of VPL for Moodle - http://vpl.dis.ulpgc.es/
//
// VPL for Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// VPL for Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with VPL for Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * List student submissions of a VPL instances
 *
 * @package mod_vpl
 * @copyright 2012 onwards Juan Carlos Rodríguez-del-Pino
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @author Juan Carlos Rodríguez-del-Pino <jcrodriguez@dis.ulpgc.es>
 */

require_once(dirname( __FILE__ ) . '/../../../config.php');

global $CFG, $OUTPUT, $USER, $PAGE;
require_once($CFG->dirroot.'/mod/vpl/locallib.php');
require_once($CFG->dirroot.'/mod/vpl/vpl.class.php');
require_once($CFG->dirroot.'/mod/vpl/vpl_submission.class.php');
require_once($CFG->dirroot.'/mod/vpl/vpl_submission_CE.class.php');

function get_this_page_url($id, $showgrades, $group, $getasstring = false) {
    $url = new moodle_url( '/mod/vpl/views/submissionslist.php', array ( 'id' => $id ) );
    if ($showgrades) {
        $url->param('showgrades', $showgrades);
    }
    if ($group) {
        $url->param('group', $group);
    }
    if ($getasstring) {
        return $url->out(false);
    } else {
        return $url;
    }
}

function vpl_get_listmenu($id, $showgrades, $group) {
    $menu = new action_menu();
    $url = new moodle_url( '/mod/vpl/views/activityworkinggraph.php', array (
            'id' => $id) );
    $menu->add(vpl_get_menu_action_link('submissionscharts', $url));
    if ($showgrades) {
        $url = get_this_page_url($id, 0, $group);
        $menu->add(vpl_get_menu_action_link('submissionslist', $url));
    } else {
        $url = get_this_page_url($id, 1, $group);
        $menu->add(vpl_get_menu_action_link('gradercomments', $url));
    }

    $url = new moodle_url( '/mod/vpl/views/downloadallsubmissions.php', array (
            'id' => $id) );
    $menu->add(vpl_get_menu_action_link('downloadsubmissions', $url));
    $url = new moodle_url( '/mod/vpl/views/downloadallsubmissions.php', array (
            'id' => $id,
            'all' => 1) );
    $menu->add(vpl_get_menu_action_link('downloadallsubmissions', $url));
    return $menu;
}

require_login();

$id = required_param( 'id', PARAM_INT );
$group = optional_param( 'group', 0, PARAM_INT );
$showgrades = optional_param( 'showgrades', 0, PARAM_INT );
$subselection = vpl_get_set_session_var( 'subselection', 'allsubmissions', 'selection' );
$vpl = new mod_vpl( $id );
$vpl->prepare_page( 'views/submissionslist.php', array (
        'id' => $id
) );

$cm = $vpl->get_course_module();
$vpl->require_capability( VPL_GRADE_CAPABILITY );
\mod_vpl\event\vpl_all_submissions_viewed::log( $vpl );

// Unblock user session.
session_write_close();

// Get graders.
$gradeable = $vpl->get_grade() != 0;

// Get students.
$currentgroup = groups_get_activity_group( $cm, true );
if (! $currentgroup) {
    $currentgroup = '';
}
if ($vpl->is_group_activity()) {
    $list = groups_get_all_groups($vpl->get_course()->id, 0, $cm->groupingid);
} else {
    $list = $vpl->get_students( $currentgroup );
}
$submissions = $vpl->all_last_user_submission();
$submissionsnumber = $vpl->get_submissions_number();
$maxgrades = $vpl->get_max_grades();

$popupoptions = array (
        'height' => 550,
        'width' => 780,
        'directories' => 0,
        'location' => 0,
        'menubar' => 0,
        'personalbar' => 0,
        'status' => 0,
        'toolbar' => 0
);



if ($vpl->is_group_activity()) {
    $strname = get_string( 'group' );
} else {
    $strname = vpl_get_name_fields_display();
}
$strsubtime = get_string( 'submittedon', VPL );
$strgrade = get_string( 'grade' );
if (!$showgrades) {
    $hrefgradedetails = get_this_page_url($id, 1, $group);
    $linkgradedetails = $OUTPUT->action_link( $hrefgradedetails, '', null,
            array( 'title' => get_string( 'gradercomments', VPL ) ),
            new pix_icon('gradercomments', '', 'mod_vpl') );
    $strgrade .= ' ' . $linkgradedetails;
}
$strgrader = get_string( 'grader', VPL );
$strgradedon = get_string( 'gradedon', VPL );
$strmaxgrade = get_string( 'maxgradeobtained', VPL );
$strcomments = get_string( 'gradercomments', VPL );
$hrefnsub = vpl_mod_href( 'views/activityworkinggraph.php', 'id', $id );
$action = new popup_action( 'click', $hrefnsub, 'activityworkinggraph' . $id, $popupoptions );
$linkworkinggraph = $OUTPUT->action_link( $hrefnsub, '', $action,
        array ( 'title' => get_string( 'submissionscharts' , VPL ) ),
        new pix_icon('submissionscharts', '', 'mod_vpl') );
$strsubmissions = get_string( 'submissions', VPL ) . ' ' . $linkworkinggraph;
$strnevaluations = get_string( 'evaluations', VPL );

$table = new html_table();

$table->head = array(
        '#',
        $strname
);

if ($showgrades) {
    $table->head = array_merge($table->head, array (
            $strgrade,
            $strcomments,
            $strgrader,
            $strgradedon
    ));
} else if ($gradeable) {
    $table->head = array_merge($table->head, array (
            $strsubtime,
            $strsubmissions,
            $strnevaluations,
            $strgrade,
            $strmaxgrade
    ));
} else {
    $table->head = array_merge($table->head, array (
            $strsubtime,
            $strsubmissions
    ));
}

$table->head[] = $OUTPUT->render(vpl_get_listmenu($id, $showgrades, $group));

$table->id = 'submissionstable';
$PAGE->requires->js_call_amd('mod_vpl/sorttable', 'makeSortable', array('submissionstable', array(0, -1), 0));

$usernumber = 0;
$ngrades = array (); // Number of revisions made by teacher.

$userslists = array(
        'notexecuted' => array(),
        'notgraded' => array(),
        'all' => array()
);

// Get all information.
foreach ($list as $uginfo) {
    $submission = null;
    $nsubmissions = null;
    $maxgradevalue = null;
    if (! isset( $submissions [$uginfo->id] )) {
        if ($subselection != 'all') {
            continue;
        }
    } else {
        $subinstance = $submissions [$uginfo->id];
        $submission = new mod_vpl_submission_CE( $vpl, $subinstance );
        $subid = $subinstance->id;
        if ($subinstance->dategraded > 0) {
            if ($subselection == 'notgraded') {
                continue;
            }
            if ($subselection == 'gradedbyuser' && $subinstance->grader != $USER->id) {
                continue;
            }
        } else {
            $subinstance->grade = null;
            if ($subselection == 'graded' || $subselection == 'gradedbyuser') {
                continue;
            }
        }
        if (isset( $submissionsnumber [$uginfo->id] )) {
            $nsubmissions = $submissionsnumber [$uginfo->id]->submissions;
        }

        if (isset( $maxgrades [$uginfo->id] )) {
            $maxgradevalue = $maxgrades [$uginfo->id]->maxgrade;
        }
    }
    $actions = new action_menu();
    // When group activity => add lastname to groupname for order porpouse.
    if ($vpl->is_group_activity()) {
        $gr = $uginfo;
        $users = $vpl->get_group_members($gr->id);
        if ( count($users) == 0 ) {
            continue;
        }
        $user = reset( $users );
        $user->firstname = '';
        $user->lastname = $gr->name;
    } else {
        $user = $uginfo;
    }
    $gradecomments = '';
    $linkparms = array('id' => $id, 'userid' => $user->id);
    $viewlink = new moodle_url('/mod/vpl/forms/submissionview.php', $linkparms);
    $actions->add(vpl_get_menu_action_link('submissionview', $viewlink));
    if ($submission == null) {
        $subtime = $OUTPUT->action_link( $viewlink, get_string( 'nosubmission', VPL ) );
        $prev = '';
        $evals = '';
        $grade = '';
        $grader = '';
        $gradedon = '';
        $maxgrade = '';
    } else {
        $subtime = $OUTPUT->action_link( $viewlink, userdate( $subinstance->datesubmitted), null,
                array(
                        'title' => get_string('submissionview', VPL),
                        'value' => $subinstance->datesubmitted
                ) );
        if ($nsubmissions !== null) {
            $prevlink = new moodle_url('/mod/vpl/views/previoussubmissionslist.php', $linkparms);
            $prev = $OUTPUT->action_link( $prevlink, $nsubmissions, null,
                    array(
                            'title' => get_string('previoussubmissionslist', VPL)
                    ) );
            $actions->add(vpl_get_menu_action_link('previoussubmissionslist', $prevlink));
        } else {
            $prev = '';
        }
        $evals = $subinstance->nevaluations;
        $subid = $subinstance->id;
        $userslists['all'][] = $user->id;
        $gradevalue = $subinstance->grade;
        $result = $submission->getCE();
        if ($result ['executed'] !== 0) {
            $prograde = $submission->proposedGrade( $result ['execution'] );
        } else {
            $userslists['notexecuted'][] = $user->id;
            $prograde = '';
        }
        if ($subinstance->dategraded > 0) {
            $gradetext = $submission->get_grade_core();
            // Add proposed grade diff.
            if ($prograde > '' && $prograde != $subinstance->grade) {
                $gradetext .= ' (' . $prograde . ')';
            }
            $graderid = $subinstance->grader;
            // Count evaluator marks.
            if (isset( $ngrades [$graderid] )) {
                $ngrades [$graderid] ++;
            } else {
                $ngrades [$graderid] = 1;
            }
            $grader = fullname( mod_vpl_submission::get_grader( $graderid ) );
            if ($showgrades) {
                $feedback = $submission->get_grade_comments();
                if ($feedback) {
                    $gradecomments = nl2br(s($feedback));
                }
            }
        } else {
            $userslists['notgraded'][] = $user->id;
            if ($prograde > '') {
                $gradetext = get_string( 'proposedgrade', VPL, $submission->get_grade_core( $prograde ) );
                $gradevalue = $prograde;
            } else {
                $gradetext = get_string( 'nograde' );
            }
            $grader = '';
        }

        $gradetext = '<div id="g' . $subid . '" class="gd' . $subid . '" value="'.$gradevalue.'">' . $gradetext . '</div>';
        if ($subinstance->grader == $USER->id || !($subinstance->dategraded > 0)) {
            $hrefgrade = vpl_mod_href( 'forms/gradesubmission.php', 'id', $id, 'userid', $user->id, 'inpopup', 1 );
            $action = new popup_action( 'click', $hrefgrade, 'gradesub' . $user->id, $popupoptions );
            $grade = $OUTPUT->action_link( $hrefgrade, $gradetext, $action,
                    array(
                            'title' => get_string('grade')
                    ) );
        } else {
            $grade = $gradetext;
        }

        if ($maxgradevalue !== null) {
            $maxgradetext = $vpl->format_grade($maxgradevalue);
            $maxgrade = '<span value="'.$maxgradevalue.'">'.$maxgradetext.'</span>';
        } else {
            $maxgrade = '';
        }

        $link = new moodle_url('/mod/vpl/forms/gradesubmission.php', $linkparms);
        $actions->add(vpl_get_menu_action_link('grade', $link, 'moodle'));
        // Add div id to submission info.
        $grader = '<div id="m' . $subid . '" class="gd' . $subid . '">' . $grader . '</div>';
        $gradedon = '<div id="o' . $subid . '" class="gd' . $subid . '">'.
                        $submission->get_gradedon_date_formatted().
                    '</div>';
    }
    $copylink = new moodle_url('/mod/vpl/forms/edit.php', array('id' => $id, 'userid' => $user->id, 'privatecopy' => 1));
    $action = new popup_action( 'click', $copylink, 'privatecopyl' . $id, $popupoptions );
    $usernumber ++;
    $usernumberlink = $OUTPUT->action_link( $copylink, $usernumber, $action,
            array(
                    'title' => get_string('copy', VPL)
            ) );
    $actions->add(vpl_get_menu_action_link('copy', $copylink));

    $submissiondata = array (
            $usernumberlink,
            '<span value="'.$user->lastname.'">' . $vpl->user_fullname_picture($user, false) . '</span>',
    );

    if ($showgrades) {
        $submissiondata = array_merge($submissiondata, array (
                $grade,
                $gradecomments,
                $grader,
                $gradedon
        ));
    } else if ($gradeable) {
        $submissiondata = array_merge($submissiondata, array (
                $subtime,
                $prev,
                $evals,
                $grade,
                $maxgrade
        ));
    } else {
        $submissiondata = array_merge($submissiondata, array (
                $subtime,
                $prev
        ));
    }

    $table->data [] = array_merge($submissiondata, array (
            $OUTPUT->render($actions)
    ));
}
if (count( $ngrades )) {
    $tablegraders = new html_table();
    $tablegraders->head = array (
            '#',
            $strgrader,
            get_string( 'grades' )
    );
    $tablegraders->align = array (
            'right',
            'left',
            'center'
    );
    $tablegraders->wrap = array (
            'nowrap',
            'nowrap',
            'nowrap'
    );
    $tablegraders->data = array ();
    $gradernumber = 0;
    foreach ($ngrades as $graderid => $marks) {
        $gradernumber ++;
        $grader = mod_vpl_submission::get_grader( $graderid );
        $picture = '';
        if ($graderid > 0) { // No automatic grading.
            $picture = $OUTPUT->user_picture( $grader );
        }
        $tablegraders->data [] = array (
                $gradernumber,
                $picture . ' ' . fullname( $grader ),
                sprintf( '%d/%d  (%.2f%%)', $marks, $usernumber, ( float ) 100.0 * $marks / $usernumber )
        );
    }
}



// Print header.
$vpl->print_header( get_string( 'submissionslist', VPL ) );
$vpl->print_view_tabs( basename( __FILE__ ) );
@ob_flush();
flush();

// Find out current groups mode.
// Menu for groups.
if ( groups_get_activity_groupmode( $cm ) || groups_get_course_groupmode( $vpl->get_course() ) ) {
    $groupsurl = get_this_page_url($id, $showgrades, 0);
    groups_print_activity_menu( $cm, $groupsurl );
}
// Print user selection by submission state.
$urlbase = get_this_page_url($id, $showgrades, $group, true) . '&selection=';
$urlindex = vpl_select_index( $urlbase, array (
        'all',
        'allsubmissions',
        'notgraded',
        'graded',
        'gradedbyuser'
) );
$urls = array_merge( array (
        $urlbase . 'all' => get_string( 'all' )
), vpl_select_array( $urlbase, array (
        'allsubmissions',
        'notgraded',
        'graded',
        'gradedbyuser'
) ) );
$urlsel = new url_select( $urls, $urlindex [$subselection], null );
$urlsel->set_label( get_string( 'submissionselection', VPL ) );
echo $OUTPUT->render( $urlsel );

// Print evaluation buttons.
if ($gradeable || $vpl->get_instance()->evaluate) {
    $returnurl = get_this_page_url($id, $showgrades, $group, true);
    $baseurl = vpl_mod_href('forms/evaluation.php', 'id', $id, 'returnurl', $returnurl);
    echo '<br>';
    echo '<span>' . get_string( 'evaluate', VPL ) . ' </span>';
    $stringmanager = get_string_manager();
    foreach ($userslists as $type => $userslist) {
        if ($stringmanager->string_exists($type, VPL)) {
            $label = get_string($type, VPL);
        } else {
            $label = get_string($type);
        }
        $label .= ' (' . count($userslist) . ')';
        $url = vpl_url_add_param($baseurl, 'userid', reset($userslist));
        $url = vpl_url_add_param($url, 'userlist', implode(',', $userslist));
        $button = new single_button(new moodle_url($url), $label, 'get');
        $button->disabled = empty($userslist);
        echo $OUTPUT->render($button);
    }
    echo '<br>';
}
echo '<br>';
@ob_flush();
flush();
echo html_writer::table( $table );
if (count( $ngrades ) > 0) {
    echo '<br>';
    echo html_writer::table( $tablegraders );
}
$vpl->print_footer_simple();
